package turtleWorld;

import grobs.* ;
import java.awt.* ;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.* ;

public class SkyWriter extends Controller  {

	
    private Turtle skye = new Turtle(Color.black) ;
    
    // The constructor
    public SkyWriter( GrobArena arena, Log log, ThreadCounter threadCounter ) {
        super( arena, log, threadCounter ) ;
        arena.add( skye ) ; }
    
    public String controllerName() {
    	return "SkyWri" ;
    }
    
    @Override
	public String[] buttons() {
    	return new String[] { "reset", "go" } ; }

    @Override
	public String[] checkBoxes() {
    	return new String[] { } ; }

    @Override
	public String[] startUp() {
    	return new String[] {
        	"reset" } ; }

    public void reset() {
        skye.reset();
        skye.clearTrail() ;
        skye.setTrailLength( Double.POSITIVE_INFINITY ) ;
        double x = arena.getWidth() / 2.0 ;
        double y = arena.getHeight() / 2.0 ;
        skye.setPosition(x, y) ;
    }
    
    public void go() {
    	Font font = new Font( Font.SERIF, Font.BOLD, 80 ) ;
    	putStr("Turtle World", 100, 100, font) ;
    	skye.penUp() ;
    	straightTo(100, 170) ;
    	skye.setOrientation(0) ;
    	skye.penDown() ;
    	straightTo(680, 170) ;
    }
    
    private void putStr(String str, double x, double y, Font font) {
    	FontMetrics fm = arena.getGraphics().getFontMetrics(font) ;
    	FontRenderContext frc = fm.getFontRenderContext() ;
    	GlyphVector gv = font.createGlyphVector(frc, str) ;
    	Shape shape = gv.getOutline() ;
    	followShape( shape, x, y ) ;
    }
    
    private void followShape( Shape shape, double x, double y ) {
    	// This is pretty gnarly.  Consult the Java docs on PathIterators

    	Rectangle2D r = shape.getBounds2D() ;
    	PathIterator iterator = shape.getPathIterator( AffineTransform.getTranslateInstance(x-r.getX(), y-r.getY())) ;
    	double coords[] = new double[6] ;
    	double closeX = 0, closeY = 0 ;  // The destination of the most recent MOVE_TO segment
    	double cpx = 0, cpy = 0 ; // The current point
    	// For each segment of the path
    	while( ! iterator.isDone() ) {
    		int kind = iterator.currentSegment(coords) ;
    		switch( kind ) {
    		case PathIterator.SEG_MOVETO:
    			//System.out.println("Move to (" +coords[0]+ ", " +coords[1]+ ")") ;
    			skye.penUp() ;
    			straightTo( coords[0], coords[1] ) ;
    			closeX = cpx = coords[0] ; closeY = cpy = coords[1] ; 
    			break ;
    		case PathIterator.SEG_LINETO:
    			//System.out.println("Line to (" +coords[0]+ ", " +coords[1]+ ")") ;
    			skye.penDown() ;
    			straightTo( coords[0], coords[1] ) ;
    			cpx = coords[0] ; cpy = coords[1] ; 
    			break ;
    		case PathIterator.SEG_CLOSE:
    			//System.out.println("Close to (" +closeX+ ", " +closeY+ ")") ;
    			skye.penDown() ;
    			straightTo( closeX, closeY ) ;
    			cpx = closeX ; cpy = closeY ; 
    			break;
    		case PathIterator.SEG_QUADTO: {
    			//System.out.println("Quad (" +coords[0]+ ", " +coords[1]+ ") (" +coords[2] +", "+ coords[3] +")") ;
    			skye.penDown() ;
    			Path path = bezierToPath( cpx, cpy, coords[0], coords[1], coords[2], coords[3] ) ;
    			followPath(path) ;
    			cpx = coords[2] ; cpy = coords[3] ; 
    			break; }
    		case PathIterator.SEG_CUBICTO: {
    			//System.out.println("Cubic (" +coords[0]+ ", " +coords[1]+ ") (" +coords[2] +", "+ coords[3] +") ("+ coords[4] +", "+ coords[5] +")") ;
    			skye.penDown() ;
    			Path path = bezierToPath( cpx, cpy, coords[0], coords[1], coords[2], coords[3], coords[4], coords[5] ) ;
    			followPath(path);
    			cpx = coords[4] ; cpy = coords[5] ;
    			break; }
    		default: monitor.Assertion.test(false) ; }
    		iterator.next() ;
    	}
    }

	static private class Path {
		// Invariant x.length == y.length && x.length > 0
    	double[] x, y ;
    	Path( double[] x, double[] y ) { this.x = x ; this.y = y ; }
    }
    
    private void followPath(Path p) {
		int n = p.x.length-1 ;
		for( int i = 0 ; i < n ; ++i ) {
			straightTo(p.x[i+1], p.y[i+1]) ;
		}
	}

	private Path bezierToPath(double p0x, double p0y, double p1x, double p1y, double p2x, double p2y, double p3x, double  p3y ) {
		// Find points on a cubic Bezier curve. 
		// See http://docs.oracle.com/javase/6/docs/api/java/awt/geom/PathIterator.html
		// for details.
		int n = 5 ;
		double[] x = new double[n+1] ;
		double[] y = new double[n+1] ;
		for( int i = 0 ; i <= n ; ++i ) {
			double t = (double) i / n ;
			x[i] = (1-t)*(1-t)*(1-t)*p0x + 3*t*(1-t)*(1-t)*p1x + t*t*(1-t)*p2x + t*t*t*p3x ; 
			y[i] = (1-t)*(1-t)*(1-t)*p0y + 3*t*(1-t)*(1-t)*p1y + t*t*(1-t)*p2y + t*t*t*p3y ; }
		return new Path(x,y) ;
	}
	
	private Path bezierToPath(double p0x, double p0y, double p1x, double p1y, double p2x, double p2y) {
		// Find points on a quadratic Bezier curve.
		// See http://docs.oracle.com/javase/6/docs/api/java/awt/geom/PathIterator.html
		// for details.
		int n = 5 ;
		double[] x = new double[n+1] ;
		double[] y = new double[n+1] ;
		for( int i = 0 ; i <= n ; ++i ) {
			double t = (double) i / n ;
			x[i] = (1-t)*(1-t)*p0x + 2*t*(1-t)*p1x + t*t*p2x ; 
			y[i] = (1-t)*(1-t)*p0y + 2*t*(1-t)*p1y + t*t*p2y ; }
		return new Path(x,y) ;
	}

	private void straightTo(double destX, double destY) {
		double x = skye.getPositionX() ;
		double y = skye.getPositionY() ;
		//System.out.println("Straight from (" +x+ ", " +y+ ") to (" +destX+ ", "+ destY +")" ) ;
		skye.setDirection(destX-x, destY-y) ;
		//System.out.println("Heading is " +crush.getOrientation() ) ;
		double dist = dist(skye.getPositionX(), skye.getPositionY(), destX, destY) ; ;
		double speed = 200 ;
		double time = dist / speed ;
		skye.setSpeed(speed) ;
		pause( time ) ;
		skye.setSpeed(0) ;
	}

	private double dist(double p0x, double p0y, double p1x, double p1y) {
		double dx = p1x-p0x , dy = p1y-p0y ;
		return Math.sqrt(dx*dx+dy*dy) ;
	}
}