package turtleWorld;

import grobs.* ;

import java.awt.* ;
import java.awt.geom.* ;

public class TurtleController0 extends Controller  {

	
    private Turtle crush = new Turtle(Color.blue) ;
    private Turtle squirt = new Turtle(Color.red) ;
    
    // The constructor
    public TurtleController0( GrobArena arena, Log log, ThreadCounter threadCounter ) {
        super( arena, log, threadCounter ) ;
        arena.add( crush ) ;
        arena.add( squirt ) ;
    }
    
    public String controllerName() {
    	return "zero" ;
    }
    
    @Override
	public String[] buttons() {
    	return new String[] {
    			"reset", "start", "stop", "faster", "slower", "penUp", "penDown",
    			"left", "right", "square", "circle", "star52", "hexagon", "octagon", "mySequence" } ; }

    @Override
	public String[] checkBoxes() {
    	return new String[] {
    		"wiggle", "bounce", "trailCheck", "changeColor"} ; }

    @Override
	public String[] startUp() {
    	return new String[] {
        	"reset" } ; }

    public void reset() {
        crush.reset();
        crush.clearTrail() ;
        crush.setTrailLength( Double.POSITIVE_INFINITY ) ;
        double x = arena.getWidth() / 2.0 ;
        double y = arena.getHeight() / 2.0 ;
        crush.setPosition(x, y) ;
    }

    public void start() {
    	crush.setSpeed( 100 ) ;
    }

    public void stop() {
    	crush.setSpeed( 0 ) ;
    }

    public void circle() {
    }
    
    void turn( double  a ) {
    	double alpha = crush.getOrientation() ;
    	crush.setOrientation( alpha + a ) ;
    }

    public void right() {
    	turn( 30 ) ;
    }

    public void left() {
    	turn( - 30 ) ;
    }

    public void mySequence() {
    }

    public void a() {
        left() ;
    }

    public void d() {
        right() ;
    }

    public void penDown() {
        crush.penDown() ;
    }

    public void penUp() {
        crush.penUp();
    }

    public void faster() {
    	double speed = crush.getSpeed() ;
    	if( speed < 80 ) {
    		crush.setSpeed( speed + 20 ) ;
    	} else {
    		crush.setSpeed( 100 ) ;
    	}
    }

    public void slower() {
    	double speed = crush.getSpeed() ;
    	if( speed > 20) {
    		crush.setSpeed( speed - 20 ) ;
    	} else {
    		crush.setSpeed( 0 ) ;
    	}
    }
    
    void polygon( int n, double length, int revolutions ) {
    	crush.penDown() ;
        crush.setSpeed(length) ;
        for (int i = 0 ; i < n ; i++) {
            pause(1) ;
            turn( 360.0 * revolutions / n ) ;
        }
        crush.setSpeed(0) ;
    }

    public void square() {
    	polygon( 4, 100, 1 ) ;
    }

    public void hexagon() {
    	polygon( 6, 80, 1 ) ;
    }

    public void octagon() {
    	polygon( 8, 60, 1 ) ;
    }
    
    public void star52( ) {
    	polygon( 5, 100, 2) ;
    }

    private boolean wiggle = false ;

    public void wiggleOff() {
        wiggle = false ; }

    public void wiggleOn() {
        wiggle = true ;
        while( wiggle ) {
            crush.setSpin( 90 );
            pause( 0.5 ) ;
            crush.setSpin( - 90 );
            pause( 0.5 ) ;
        }
        crush.setSpin( 0 ) ;
    }

    private boolean bounce = false ;

    public void bounceOff( ) {
        bounce = false ; }

    public void bounceOn( ) {
        bounce = true ;
        while( bounce ) {
            pause(0.1) ;
            bouncer( crush ) ;
        }
    }
    
    private void bouncer( Turtle t ) {
        double rightEdge = arena.getWidth() ;
        double bottomEdge = arena.getHeight() ;
        double x = t.getPositionX() ;
        double y = t.getPositionY() ;
        double vx = t.getVelocityX() ;
        double vy = t.getVelocityY() ;
        double radius = t.getRadius() ;

        if( x < radius && vx < 0 ) {
            t.setVelocity(- vx, vy );
        }
        if( x > rightEdge-radius && vx > 0 ) {
            t.setVelocity(- vx, vy );
        }
        if( y < radius && vy < 0 ) {
            t.setVelocity( vx, -vy );
        }
        if( y > bottomEdge-radius && vy > 0 ) {
            t.setVelocity( vx, -vy );
        }
    }
    
    private boolean trailCheck = false ;

    public void trailCheckOff( ) {
    	trailCheck = false ; }

    public void trailCheckOn( ) {
    	crush.penDown() ;
    	int length = 200 ;
    	trailCheck = true ;
    	int i = 0 ;
        while( trailCheck ) {
        	crush.setTrailLength( length ) ;
        	length += 20 ;
        	if( crush.doesTrailIntersectTrail( crush ))
        		log.println( "Intersect " + i++) ;
        	pause( 1 ) ;
        	}
        }
    
    public void radarTest() {
    	Turtle secondTurtle = new Turtle( Color.RED ) ;
    	secondTurtle.setPosition(300, 300) ;
    	arena.add(secondTurtle) ;
    	Shape circle = new Ellipse2D.Double(200, 200, 50, 50 ) ;
    	ShapeGrob circleGrob = new ShapeGrob( circle ) ;
    	arena.add( circleGrob ) ;
    	crush.setSpeed(100) ;
    	crush.setRadarRange(40) ;
    	while( true ) {
    		if( crush.checkRadarFor( secondTurtle ) ) {
    			log.print( "second turtle" ) ;
    		}
    		if( crush.checkRadarFor( circleGrob ) ) {
    			log.print( "circle" ) ;
    		}
    		pause( 0.1 ) ;
    	}
    }
    
    boolean changeColor ;
    
    public void changeColorOn() {
    	changeColor = true ;
    	while( changeColor ) {
    		pause( 0.25 ) ;
    		crush.setFillColor(Color.GRAY) ;
    		pause( 0.25 ) ;
    		crush.setFillColor(Color.GREEN) ;
    		pause( 0.25 ) ;
    		crush.setFillColor(Color.BLUE) ;	
    	}
    }
    
    public void changeColorOff() {
    	changeColor = false ;
    }
}